@@ -115,14 +115,6 @@ gchar *nemo_get_drive_icon_name (GDrive *drive);
 gchar *nemo_get_best_guess_file_mimetype (const gchar *filename,
                                           GFileInfo   *info,
                                           goffset      size);
-void nemo_query_btime_async (GFile               *file,
-                             GCancellable        *cancellable,
-                             GAsyncReadyCallback  callback,
-                             gpointer             user_data);
-
-time_t nemo_query_btime_finish (GFile         *file,
-                                GAsyncResult  *res,
-                                GError       **error);
 
 gboolean nemo_treating_root_as_normal (void);
 gboolean nemo_user_is_root (void);
