@@ -340,6 +340,7 @@ nemo_icon_info_lookup (GIcon *icon,
                                                                  GTK_ICON_LOOKUP_FORCE_SIZE);
 
             pixbuf = gtk_icon_info_load_icon (gtkicon_info, NULL);
+            g_object_unref (gtkicon_info);
         }
 
         icon_info = nemo_icon_info_new_for_pixbuf (pixbuf, scale);
@@ -610,11 +611,12 @@ nemo_get_icon_text_width_for_zoom_level (NemoZoomLevel  zoom_level)
         return NEMO_ICON_TEXT_WIDTH_LARGEST;
     case NEMO_ZOOM_LEVEL_NULL:
     default:
-        g_return_val_if_reached (NEMO_ICON_SIZE_STANDARD);
+        g_return_val_if_reached (NEMO_ICON_TEXT_WIDTH_STANDARD);
     }
 }
 
 
+
 guint
 nemo_get_desktop_icon_size_for_zoom_level (NemoZoomLevel zoom_level)
 {
@@ -637,6 +639,26 @@ nemo_get_desktop_icon_size_for_zoom_level (NemoZoomLevel zoom_level)
     }
 }
 
+guint
+nemo_get_desktop_text_width_for_zoom_level (NemoZoomLevel  zoom_level)
+{
+    switch (zoom_level) {
+    case NEMO_ZOOM_LEVEL_SMALLER:
+        return NEMO_DESKTOP_TEXT_WIDTH_SMALLER;
+    case NEMO_ZOOM_LEVEL_SMALL:
+        return NEMO_DESKTOP_TEXT_WIDTH_SMALL;
+    case NEMO_ZOOM_LEVEL_STANDARD:
+        return NEMO_DESKTOP_TEXT_WIDTH_STANDARD;
+    case NEMO_ZOOM_LEVEL_LARGE:
+        return NEMO_DESKTOP_TEXT_WIDTH_LARGE;
+    case NEMO_ZOOM_LEVEL_LARGER:
+        return NEMO_DESKTOP_TEXT_WIDTH_LARGER;
+    case NEMO_ZOOM_LEVEL_NULL:
+    default:
+        g_return_val_if_reached (NEMO_DESKTOP_TEXT_WIDTH_STANDARD);
+    }
+}
+
 guint
 nemo_get_list_icon_size_for_zoom_level (NemoZoomLevel zoom_level)
 {
