@@ -56,11 +56,11 @@ static int		     name_field_changed_signal_id;
 static GtkWidget	    *remove_button = NULL;
 static GtkWidget            *jump_button = NULL;
 static GtkWidget            *sort_button = NULL;
+static GtkWidget            *close_button = NULL;
 static gboolean		     text_changed = FALSE;
 static gboolean		     name_text_changed = FALSE;
 static GtkWidget	    *uri_field = NULL;
 static int		     uri_field_changed_signal_id;
-static int		     row_changed_signal_id;
 static int		     row_deleted_signal_id;
 static int                   row_activated_signal_id;
 static int                   button_pressed_signal_id;
@@ -83,10 +83,6 @@ static void     on_jump_button_clicked                      (GtkButton
 							     gpointer              user_data);
 static void     on_sort_button_clicked                      (GtkButton            *button,
 							     gpointer              user_data);
-static void	on_row_changed				    (GtkListStore	  *store,
-							     GtkTreePath	  *path,
-							     GtkTreeIter	  *iter,
-							     gpointer		   user_data);
 static void	on_row_deleted				    (GtkListStore	  *store,
 							     GtkTreePath	  *path,
 							     gpointer		   user_data);
@@ -115,7 +111,8 @@ static void     on_window_hide_event                        (GtkWidget
 							     gpointer              user_data);
 static void     on_window_destroy_event                     (GtkWidget            *widget,
 							     gpointer              user_data);
-static void     repopulate                                  (void);
+static void     repopulate_now                                  (void);
+static void     queue_repopulate                                  (void);
 static void     set_up_close_accelerator                    (GtkWidget            *window);
 static void	open_selected_bookmark 			    (gpointer   user_data, GdkScreen *screen);
 static void	update_bookmark_from_text		    (void);
@@ -129,7 +126,8 @@ static void	update_bookmark_from_text		    (void);
 #define BOOKMARK_LIST_COLUMN_NAME		1
 #define BOOKMARK_LIST_COLUMN_BOOKMARK		2
 #define BOOKMARK_LIST_COLUMN_STYLE		3
-#define BOOKMARK_LIST_COLUMN_COUNT		4
+#define BOOKMARK_LIST_COLUMN_IS_SEPARATOR 4
+#define BOOKMARK_LIST_COLUMN_COUNT		5
 
 /* layout constants */
 
@@ -139,49 +137,27 @@ static void	update_bookmark_from_text		    (void);
 
 /* Larger size initially; user can stretch or shrink (but not shrink below min) */
 #define BOOKMARKS_WINDOW_INITIAL_WIDTH	500
-#define BOOKMARKS_WINDOW_INITIAL_HEIGHT	200
+#define BOOKMARKS_WINDOW_INITIAL_HEIGHT	300
 
 static void
 nemo_bookmarks_window_response_callback (GtkDialog *dialog,
 					     int response_id,
 					     gpointer callback_data)
 {
-	if (response_id == GTK_RESPONSE_HELP) {
-		GError *error = NULL;
-
-		gtk_show_uri (gtk_window_get_screen (GTK_WINDOW (dialog)),
-			      "help:gnome-help/nemo-bookmarks-edit",
-			      gtk_get_current_event_time (), &error);
-
-		if (error) {
-			GtkWidget *err_dialog;
-			err_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
-							     GTK_DIALOG_DESTROY_WITH_PARENT,
-							     GTK_MESSAGE_ERROR,
-							     GTK_BUTTONS_OK,
-							     _("There was an error displaying help: \n%s"),
-							     error->message);
-
-			g_signal_connect (G_OBJECT (err_dialog),
-					  "response", G_CALLBACK (gtk_widget_destroy),
-					  NULL);
-			gtk_window_set_resizable (GTK_WINDOW (err_dialog), FALSE);
-			gtk_widget_show (err_dialog);
-			g_error_free (error);
-		}
-	} else if (response_id == GTK_RESPONSE_CLOSE) {
+    if (response_id == GTK_RESPONSE_CLOSE) {
 		gtk_widget_hide (GTK_WIDGET (dialog));
-        }
+    }
 }
 
 static GtkListStore *
 create_bookmark_store (void)
 {
 	return gtk_list_store_new (BOOKMARK_LIST_COLUMN_COUNT,
-				   G_TYPE_ICON,
+				   G_TYPE_STRING,
 				   G_TYPE_STRING,
 				   G_TYPE_OBJECT,
-				   PANGO_TYPE_STYLE);
+				   PANGO_TYPE_STYLE,
+                   G_TYPE_BOOLEAN);
 }
 
 static void
@@ -274,7 +250,13 @@ create_bookmarks_window (NemoBookmarkList *list, GObject *undo_manager_source)
 	bookmark_list_widget = (GtkTreeView *)gtk_builder_get_object (builder, "bookmark_tree_view");
 	remove_button = (GtkWidget *)gtk_builder_get_object (builder, "bookmark_delete_button");
 	jump_button = (GtkWidget *)gtk_builder_get_object (builder, "bookmark_jump_button");
-	sort_button = (GtkWidget *)gtk_builder_get_object (builder, "bookmark_sort_button");
+    sort_button = (GtkWidget *)gtk_builder_get_object (builder, "bookmark_sort_button");
+	close_button = (GtkWidget *)gtk_builder_get_object (builder, "bookmark_close_button");
+
+    gtk_button_set_image (GTK_BUTTON (remove_button), gtk_image_new_from_icon_name ("list-remove-symbolic", GTK_ICON_SIZE_BUTTON));
+    gtk_button_set_image (GTK_BUTTON (jump_button), gtk_image_new_from_icon_name ("go-jump-symbolic", GTK_ICON_SIZE_BUTTON));
+    gtk_button_set_image (GTK_BUTTON (sort_button), gtk_image_new_from_icon_name ("view-sort-ascending-symbolic", GTK_ICON_SIZE_BUTTON));
+    gtk_button_set_image (GTK_BUTTON (close_button), gtk_image_new_from_icon_name ("window-close-symbolic", GTK_ICON_SIZE_BUTTON));
 
 	set_up_close_accelerator (window);
 	nemo_undo_share_undo_manager (G_OBJECT (window), undo_manager_source);
@@ -290,7 +272,7 @@ create_bookmarks_window (NemoBookmarkList *list, GObject *undo_manager_source)
 	rend = gtk_cell_renderer_pixbuf_new ();
 	col = gtk_tree_view_column_new_with_attributes ("Icon", 
 							rend,
-							"gicon", 
+							"icon-name", 
 							BOOKMARK_LIST_COLUMN_ICON,
 							NULL);
 	gtk_tree_view_append_column (bookmark_list_widget,
@@ -344,9 +326,6 @@ create_bookmarks_window (NemoBookmarkList *list, GObject *undo_manager_source)
 	bookmark_list_changed_signal_id =
 		g_signal_connect (bookmarks, "changed",
 				  G_CALLBACK (on_bookmark_list_changed), NULL);
-	row_changed_signal_id =
-		g_signal_connect (bookmark_list_store, "row_changed",
-				  G_CALLBACK (on_row_changed), NULL);
 	row_deleted_signal_id =
 		g_signal_connect (bookmark_list_store, "row_deleted",
 				  G_CALLBACK (on_row_deleted), NULL);
@@ -401,7 +380,7 @@ create_bookmarks_window (NemoBookmarkList *list, GObject *undo_manager_source)
 	gtk_tree_selection_set_mode (bookmark_selection, GTK_SELECTION_BROWSE);
 	
 	/* Fill in list widget with bookmarks, must be after signals are wired up. */
-	repopulate();
+	repopulate_now();
 
 	g_object_unref (builder);
 	
@@ -436,15 +415,25 @@ edit_bookmarks_dialog_set_signals (GObject *undo_manager_source)
 static NemoBookmark *
 get_selected_bookmark (void)
 {
+    NemoBookmark *bookmark;
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+
 	g_return_val_if_fail(NEMO_IS_BOOKMARK_LIST(bookmarks), NULL);
 
-	if (!get_selection_exists())
-		return NULL;
+    model = GTK_TREE_MODEL (bookmark_list_store);
 
-	if (nemo_bookmark_list_length (bookmarks) < 1)
-		return NULL;
+    if (!gtk_tree_selection_get_selected (bookmark_selection,
+                                          &model,
+                                          &iter)) {
+        return NULL;
+    }
 
-	return nemo_bookmark_list_item_at(bookmarks, get_selected_row ());
+    gtk_tree_model_get (GTK_TREE_MODEL (bookmark_list_store), &iter,
+                        BOOKMARK_LIST_COLUMN_BOOKMARK, &bookmark,
+                        -1);
+
+    return bookmark;
 }
 
 static guint
@@ -458,10 +447,13 @@ get_selected_row (void)
 	g_assert (get_selection_exists());
 	
 	model = GTK_TREE_MODEL (bookmark_list_store);
-	gtk_tree_selection_get_selected (bookmark_selection,
-					 &model,
-					 &iter);
-	
+
+    if (!gtk_tree_selection_get_selected (bookmark_selection,
+                                          &model,
+                                          &iter)) {
+        return -1;
+    }
+
 	path = gtk_tree_model_get_path (model, &iter);
 	indices = gtk_tree_path_get_indices (path);
 	row = indices[0];
@@ -524,12 +516,12 @@ nemo_bookmarks_window_save_geometry (GtkWindow *window)
 }
 
 static void
-on_bookmark_list_changed (NemoBookmarkList *bookmarks, gpointer data)
+on_bookmark_list_changed (NemoBookmarkList *bmarks, gpointer data)
 {
-	g_return_if_fail (NEMO_IS_BOOKMARK_LIST (bookmarks));
+	g_return_if_fail (NEMO_IS_BOOKMARK_LIST (bmarks));
 
 	/* maybe add logic here or in repopulate to save/restore selection */
-	repopulate ();
+	queue_repopulate ();
 }
 
 static void
@@ -604,37 +596,29 @@ on_sort_button_clicked (GtkButton *button,
 static void
 bookmarks_delete_bookmark (void)
 {
+    NemoBookmark *bookmark;
 	GtkTreeIter iter;
-	GtkTreePath *path;
-	gint *indices, row, rows;
+	gint i;
 
 	g_assert (GTK_IS_TREE_VIEW (bookmark_list_widget));
 	
 	if (!gtk_tree_selection_get_selected (bookmark_selection, NULL, &iter))
 		return;
 
-	/* Remove the selected item from the list store. on_row_deleted() will
-	   remove it from the bookmark list. */
-	path = gtk_tree_model_get_path (GTK_TREE_MODEL (bookmark_list_store),
-					&iter);
-	indices = gtk_tree_path_get_indices (path);
-	row = indices[0];
-	gtk_tree_path_free (path);
+    gtk_tree_model_get (GTK_TREE_MODEL (bookmark_list_store), &iter,
+                        BOOKMARK_LIST_COLUMN_BOOKMARK, &bookmark,
+                        -1);
 
-	gtk_list_store_remove (bookmark_list_store, &iter);
+    for (i = 0; i < nemo_bookmark_list_length (bookmarks); i++) {
+        NemoBookmark *bookmark_in_list = nemo_bookmark_list_item_at (bookmarks, i);
 
-	/* Try to select the same row, or the last one in the list. */
-	rows = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (bookmark_list_store), NULL);
-	if (row >= rows)
-		row = rows - 1;
+        if (bookmark_in_list == bookmark) {
+            nemo_bookmark_list_delete_item_at (bookmarks, i);
+            break;
+        }
+    }
 
-	if (row < 0) {
-		bookmarks_set_empty (TRUE);
-	} else {
-		gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (bookmark_list_store),
-					       &iter, NULL, row);
-		gtk_tree_selection_select_iter (bookmark_selection, &iter);
-	}
+    queue_repopulate ();
 }
 
 static void
@@ -644,54 +628,47 @@ on_remove_button_clicked (GtkButton *button,
         bookmarks_delete_bookmark ();
 }
 
-
-/* This is a bit of a kludge to get DnD to work. We check if the row in the
-   GtkListStore matches the one in the bookmark list. If it doesn't, we assume
-   the bookmark has just been dragged here and we insert it into the bookmark
-   list. */
 static void
-on_row_changed (GtkListStore *store,
-		GtkTreePath *path,
-		GtkTreeIter *iter,
-		gpointer user_data)
+on_row_deleted (GtkListStore *store,
+        GtkTreePath *path,
+        gpointer user_data)
 {
-	NemoBookmark *bookmark = NULL, *bookmark_in_list;
-	gint *indices, row;
-	gboolean insert_bookmark = TRUE;
+	NemoBookmark *bookmark = NULL;
 
 	store = bookmark_list_store;
+    GtkTreeIter row_iter;
+    gint row = 0, new_index = 0;
 
-	indices = gtk_tree_path_get_indices (path);
-	row = indices[0];
-	gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
-			    BOOKMARK_LIST_COLUMN_BOOKMARK, &bookmark,
-			    -1);
+    g_signal_handler_block (bookmarks,
+                            bookmark_list_changed_signal_id);
 
-	/* If the bookmark in the list doesn't match the changed one, it must
-	   have been dragged here, so we insert it into the list. */
-	if (row < (gint) nemo_bookmark_list_length (bookmarks)) {
-		bookmark_in_list = nemo_bookmark_list_item_at (bookmarks,
-								   row);
-		if (bookmark_in_list == bookmark)
-			insert_bookmark = FALSE;
-	}
+    if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &row_iter)) {
+        do {
+            gtk_tree_model_get (GTK_TREE_MODEL (store), &row_iter,
+                                BOOKMARK_LIST_COLUMN_BOOKMARK, &bookmark,
+                                -1);
+            if (bookmark != NULL) {
+                gint i;
 
-	if (insert_bookmark) {
-		g_signal_handler_block (bookmarks,
-					bookmark_list_changed_signal_id);
-		nemo_bookmark_list_insert_item (bookmarks, bookmark, row);
-		g_signal_handler_unblock (bookmarks,
-					  bookmark_list_changed_signal_id);
+                for (i = 0; i < nemo_bookmark_list_length (bookmarks); i++) {
+                    NemoBookmark *old_bm = nemo_bookmark_list_item_at (bookmarks, i);
 
-		/* The bookmark will be copied when inserted into the list, so
-		   we have to update the pointer in the list store. */
-		bookmark = nemo_bookmark_list_item_at (bookmarks, row);
-		g_signal_handler_block (store, row_changed_signal_id);
-		gtk_list_store_set (store, iter,
-				    BOOKMARK_LIST_COLUMN_BOOKMARK, bookmark,
-				    -1);
-		g_signal_handler_unblock (store, row_changed_signal_id);
-	}
+                    if (old_bm == bookmark) {
+                        nemo_bookmark_list_move_item (bookmarks, i, new_index++);
+                    }
+                }
+            } else {
+                g_settings_set_int (nemo_window_state, NEMO_PREFERENCES_SIDEBAR_BOOKMARK_BREAKPOINT, row);
+            }
+
+            row++;
+        } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &row_iter));
+    }
+
+    g_signal_handler_unblock (bookmarks,
+                              bookmark_list_changed_signal_id);
+
+    queue_repopulate ();
 }
 
 /* The update_bookmark_from_text() calls in the
@@ -743,21 +720,6 @@ on_row_activated (GtkTreeView       *view,
 	open_selected_bookmark (user_data, screen);
 }
 
-static void
-on_row_deleted (GtkListStore *store,
-		GtkTreePath *path,
-		gpointer user_data)
-{
-	gint *indices, row;
-
-	indices = gtk_tree_path_get_indices (path);
-	row = indices[0];
-
-	g_signal_handler_block (bookmarks, bookmark_list_changed_signal_id);
-	nemo_bookmark_list_delete_item_at (bookmarks, row);
-	g_signal_handler_unblock (bookmarks, bookmark_list_changed_signal_id);
-}
-
 static void
 on_selection_changed (GtkTreeSelection *treeselection,
 		      gpointer user_data)
@@ -810,7 +772,7 @@ update_bookmark_from_text (void)
 	if (text_changed) {
 		NemoBookmark *bookmark, *bookmark_in_list;
 		const char *name;
-		GIcon *icon;
+		gchar *icon_name;
 		guint selected_row;
 		GtkTreeIter iter;
 		GFile *location;
@@ -827,12 +789,20 @@ update_bookmark_from_text (void)
 		
 		bookmark = nemo_bookmark_new (location,
 						  name_text_changed ? gtk_entry_get_text (GTK_ENTRY (name_field)) : NULL,
-						  NULL);
+						  NULL, NULL);
 		
 		g_object_unref (location);
 
 		selected_row = get_selected_row ();
 
+        if (selected_row < 0) {
+            return;
+        }
+
+        if (selected_row > g_settings_get_int (nemo_window_state, NEMO_PREFERENCES_SIDEBAR_BOOKMARK_BREAKPOINT)) {
+            selected_row--;
+        }
+
 		/* turn off list updating 'cuz otherwise the list-reordering code runs
 		 * after repopulate(), thus reordering the correctly-ordered list.
 		 */
@@ -848,24 +818,20 @@ update_bookmark_from_text (void)
 		   store. */
 		gtk_tree_selection_get_selected (bookmark_selection,
 						 NULL, &iter);
-		g_signal_handler_block (bookmark_list_store,
-					row_changed_signal_id);
 
 		bookmark_in_list = nemo_bookmark_list_item_at (bookmarks,
 								   selected_row);
 
 		name = nemo_bookmark_get_name (bookmark_in_list);
-		icon = nemo_bookmark_get_icon (bookmark_in_list);
+		icon_name = nemo_bookmark_get_icon_name (bookmark_in_list);
 
 		gtk_list_store_set (bookmark_list_store, &iter,
 				    BOOKMARK_LIST_COLUMN_BOOKMARK, bookmark_in_list,
 				    BOOKMARK_LIST_COLUMN_NAME, name,
-				    BOOKMARK_LIST_COLUMN_ICON, icon,
+				    BOOKMARK_LIST_COLUMN_ICON, icon_name,
 				    -1);
-		g_signal_handler_unblock (bookmark_list_store,
-					  row_changed_signal_id);
 
-		g_object_unref (icon);
+		g_free (icon_name);
 	}
 }
 
@@ -943,13 +909,29 @@ on_window_destroy_event (GtkWidget *widget,
 }
 
 static void
-repopulate (void)
+add_breakpoint (GtkListStore *store,
+                GtkTreeIter *iter)
+{
+    gtk_list_store_append (store, iter);
+    gtk_list_store_set (store, iter,
+                        BOOKMARK_LIST_COLUMN_ICON, NULL,
+                        BOOKMARK_LIST_COLUMN_NAME, "------------------------",
+                        BOOKMARK_LIST_COLUMN_BOOKMARK, NULL,
+                        BOOKMARK_LIST_COLUMN_STYLE, PANGO_STYLE_NORMAL,
+                        BOOKMARK_LIST_COLUMN_IS_SEPARATOR, TRUE,
+                        -1);
+}
+
+static void
+repopulate_now (void)
 {
 	NemoBookmark *selected;
 	GtkListStore *store;
 	GtkTreePath *path;
 	GtkTreeRowReference *reference;
 	guint index;
+    gint breakpoint, bookmarks_length;
+    gboolean breakpoint_added;
 
 	g_assert (GTK_IS_TREE_VIEW (bookmark_list_widget));
 	g_assert (NEMO_IS_BOOKMARK_LIST (bookmarks));
@@ -981,43 +963,68 @@ repopulate (void)
 				  row_deleted_signal_id);
 	g_signal_handler_unblock (bookmark_selection,
 				  selection_changed_id);
-	
-	/* Fill the list in with the bookmark names. */
-	g_signal_handler_block (store, row_changed_signal_id);
+
+    bookmarks_length = nemo_bookmark_list_length (bookmarks);
+    breakpoint = g_settings_get_int (nemo_window_state, NEMO_PREFERENCES_SIDEBAR_BOOKMARK_BREAKPOINT);
+
+    if (breakpoint < 0) {     // Default gsettings value is -1 (which translates to 'not previously set')
+        breakpoint = bookmarks_length;
+        g_settings_set_int (nemo_window_state, NEMO_PREFERENCES_SIDEBAR_BOOKMARK_BREAKPOINT, breakpoint);
+    }
 
 	reference = NULL;
+    index = 0;
+    breakpoint_added = FALSE;
 
-	for (index = 0; index < nemo_bookmark_list_length (bookmarks); ++index) {
+    while (index < bookmarks_length) {
 		NemoBookmark *bookmark;
 		const char       *bookmark_name;
-		GIcon            *bookmark_icon;
+		gchar            *bookmark_icon;
 		GtkTreeIter       iter;
 
-		bookmark = nemo_bookmark_list_item_at (bookmarks, index);
-		bookmark_name = nemo_bookmark_get_name (bookmark);
-		bookmark_icon = nemo_bookmark_get_icon (bookmark);
+        if (index == breakpoint && !breakpoint_added) {
+            bookmark_icon = NULL;
+            bookmark = NULL;
 
-		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter, 
-				    BOOKMARK_LIST_COLUMN_ICON, bookmark_icon,
-				    BOOKMARK_LIST_COLUMN_NAME, bookmark_name,
-				    BOOKMARK_LIST_COLUMN_BOOKMARK, bookmark,
-				    BOOKMARK_LIST_COLUMN_STYLE, PANGO_STYLE_NORMAL,
-				    -1);
+            add_breakpoint (store, &iter);
+
+            breakpoint_added = TRUE;
+        } else {
+            bookmark = nemo_bookmark_list_item_at (bookmarks, index);
+            bookmark_name = nemo_bookmark_get_name (bookmark);
+            bookmark_icon = nemo_bookmark_get_icon_name (bookmark);
+
+            gtk_list_store_append (store, &iter);
+            gtk_list_store_set (store, &iter,
+                                BOOKMARK_LIST_COLUMN_ICON, bookmark_icon,
+                                BOOKMARK_LIST_COLUMN_NAME, bookmark_name,
+                                BOOKMARK_LIST_COLUMN_BOOKMARK, bookmark,
+                                BOOKMARK_LIST_COLUMN_STYLE, PANGO_STYLE_NORMAL,
+                                BOOKMARK_LIST_COLUMN_IS_SEPARATOR, FALSE,
+                                -1);
+        }
 
 		if (bookmark == selected) {
 			/* save old selection */
-			GtkTreePath *path;
+			GtkTreePath *pth;
 
-			path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
-			reference = gtk_tree_row_reference_new (GTK_TREE_MODEL (store), path);
-			gtk_tree_path_free (path);
+			pth = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
+			reference = gtk_tree_row_reference_new (GTK_TREE_MODEL (store), pth);
+			gtk_tree_path_free (pth);
 		}
 
-		g_object_unref (bookmark_icon);
+		g_free (bookmark_icon);
+
+        if (bookmark) {
+            index++;
+        }
 	}
 
-	g_signal_handler_unblock (store, row_changed_signal_id);
+    if (!breakpoint_added) {
+        GtkTreeIter iter;
+
+        add_breakpoint (store, &iter);
+    }
 
 	if (reference != NULL) {
 		/* restore old selection */
@@ -1032,6 +1039,12 @@ repopulate (void)
 
 		path = gtk_tree_row_reference_get_path (reference);
 		gtk_tree_selection_select_path (bookmark_selection, path);
+
+        gtk_tree_view_scroll_to_cell (bookmark_list_widget,
+                                      path,
+                                      NULL,
+                                      TRUE, 0.5, 0);
+
 		gtk_tree_row_reference_free (reference);
 		gtk_tree_path_free (path);
 
@@ -1041,6 +1054,19 @@ repopulate (void)
 	bookmarks_set_empty (index == 0);	  
 }
 
+static gboolean
+idle_repopulate_cb (gpointer data)
+{
+    repopulate_now ();
+
+    return G_SOURCE_REMOVE;
+}
+
+static void
+queue_repopulate (void) {
+    g_idle_add ((GSourceFunc) idle_repopulate_cb, NULL);
+}
+
 static int
 handle_close_accelerator (GtkWindow *window, 
 			  GdkEventKey *event, 
