@@ -7,7 +7,7 @@
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
 
-#include "libnemo-private/nemo-action-manager.h"
+#include "nemo-window.h"
 
 G_BEGIN_DECLS
 
@@ -15,6 +15,11 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (NemoDesktopManager, nemo_desktop_manager, NEMO, DESKTOP_MANAGER, GObject)
 
+typedef enum {
+    DESKTOP_ARRANGE_VERTICAL,
+    DESKTOP_ARRANGE_HORIZONTAL
+} NemoDesktopLayoutDirection;
+
 NemoDesktopManager* nemo_desktop_manager_get (void);
 
 gboolean nemo_desktop_manager_has_desktop_windows (NemoDesktopManager *manager);
@@ -36,6 +41,17 @@ void     nemo_desktop_manager_get_margins             (NemoDesktopManager *manag
                                                        gint               *top,
                                                        gint               *bottom);
 
+GtkWindow *nemo_desktop_manager_get_window_for_monitor  (NemoDesktopManager *manager,
+                                                         gint                monitor);
+void nemo_desktop_manager_get_overlay_info              (NemoDesktopManager *manager,
+                                                         gint                monitor,
+                                                         GtkActionGroup    **action_group,
+                                                         gint               *h_adjust,
+                                                         gint               *v_adjust);
+void     nemo_desktop_manager_show_desktop_overlay    (NemoDesktopManager *manager,
+                                                       gint                initial_monitor);
+gboolean nemo_desktop_manager_get_is_cinnamon         (NemoDesktopManager *manager);
+
 G_END_DECLS
 
 #endif /* _NEMO_DESKTOP_MANAGER_H */
