@@ -1024,7 +1024,6 @@ action_icon_view_callback (GtkAction *action,
     window = NEMO_WINDOW (user_data);
 
     set_content_view_type (window, NEMO_ICON_VIEW_ID);
-    toolbar_set_view_button (ICON_VIEW, nemo_window_get_active_pane(window));
 }
 
 
@@ -1037,7 +1036,6 @@ action_list_view_callback (GtkAction *action,
     window = NEMO_WINDOW (user_data);
 
     set_content_view_type (window, NEMO_LIST_VIEW_ID);
-    toolbar_set_view_button (LIST_VIEW, nemo_window_get_active_pane(window));
 }
 
 
@@ -1050,7 +1048,6 @@ action_compact_view_callback (GtkAction *action,
     window = NEMO_WINDOW (user_data);
 
     set_content_view_type (window, FM_COMPACT_VIEW_ID);
-    toolbar_set_view_button (COMPACT_VIEW, nemo_window_get_active_pane(window));
 }
 
 guint
@@ -1068,15 +1065,14 @@ action_for_view_id (const char *view_id)
 }
 
 void
-toolbar_set_view_button (guint action_id, NemoWindowPane *pane)
+toolbar_set_view_button (guint action_id, NemoWindow *window)
 {
     GtkAction *action, *action1, *action2;
     GtkActionGroup *action_group;
     if (action_id == NULL_VIEW) {
         return;
     }
-    action_group = nemo_window_pane_get_toolbar_action_group (pane);
-
+    action_group = nemo_window_pane_get_toolbar_action_group (nemo_window_get_active_pane (window));
 
     action = gtk_action_group_get_action(action_group,
                                          NEMO_ACTION_ICON_VIEW);
@@ -1090,20 +1086,20 @@ toolbar_set_view_button (guint action_id, NemoWindowPane *pane)
                          0, 0,
                          NULL,
                          action_icon_view_callback,
-                         NULL);
+                         window);
 
     g_signal_handlers_block_matched (action1,
                          G_SIGNAL_MATCH_FUNC,
                          0, 0,
                          NULL,
                          action_list_view_callback,
-                         NULL);
+                         window);
     g_signal_handlers_block_matched (action2,
                          G_SIGNAL_MATCH_FUNC,
                          0, 0,
                          NULL,
                          action_compact_view_callback,
-                         NULL);
+                         window);
 
     if (action_id != ICON_VIEW) {
         gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), FALSE);
@@ -1128,7 +1124,7 @@ toolbar_set_view_button (guint action_id, NemoWindowPane *pane)
                            0, 0,
                            NULL,
                            action_icon_view_callback,
-                           NULL);
+                           window);
 
 
     g_signal_handlers_unblock_matched (action1,
@@ -1136,7 +1132,7 @@ toolbar_set_view_button (guint action_id, NemoWindowPane *pane)
                            0, 0,
                            NULL,
                            action_list_view_callback,
-                           NULL);
+                           window);
 
 
     g_signal_handlers_unblock_matched (action2,
@@ -1144,7 +1140,7 @@ toolbar_set_view_button (guint action_id, NemoWindowPane *pane)
                            0, 0,
                            NULL,
                            action_compact_view_callback,
-                           NULL);
+                           window);
 
 }
 
